﻿using System;
using System.IO;
using HeuristicLab.CommandLineInterface;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab {

  public enum IsolationType {
    Docker,
    Native
  }

  [CommandLineInterface.Application(
    "HL", "1.0.0.0",
    SubCommands = new Type[] {
      typeof(OptimizeCommand),
      typeof(InspectCommand)
    }
  )]
  class ApplicationCommand : ICommand {

    [Option(Description = "Sets the path for the binaries.")]
    public string BasePath { get; set; }

    [Option(Shortcut = "i", Description = "Sets the isolation type. Possible types are: NATIVE or DOCKER.")]
    public IsolationType Isolation { get; set; }

    [Option]
    private bool StartAsRunnerHost { get; set; }

    public void Execute() {
      if (StartAsRunnerHost) {
        StartupRunnerHost();
      } else {
        SetupIsolation();
      }
    }

    private void StartupRunnerHost() {
      Stream stdin = Console.OpenStandardInput();
      var message = (TransportRunnerMessage)RunnerMessage.ReadMessageFromStream(stdin);
      IRunner runner = message.Runner;

      if (runner != null)
        runner.Run();
      else
        Console.Error.WriteLine("Cannot deserialize data from stdin to a type of RunnerConfig!");
    }

    private void SetupIsolation() {
      switch (Isolation) {
        case IsolationType.Native: Settings.Host = new NativeRunnerHost(); break;
        case IsolationType.Docker: Settings.Host = new DockerRunnerHost(); break;
        default: throw new ArgumentException("Invalid value for isolation.", "Isolation");
      }

      if (BasePath != null && !Directory.Exists(BasePath))
        throw new DirectoryNotFoundException("Path for binaries is not a directory or does not exists.");
      else BasePath = Directory.GetCurrentDirectory();

      IPluginLoader validator = PluginLoaderFactory.Create();
      Settings.assemblyInfos = validator.Validate(BasePath);
    }
  }
}
